from enum import StrEnum


class EnumMerchantCategory(StrEnum):
    """Merchant category enumeration"""

    HEALTH = "health"
    GENERAL = "general"
    SERVICES = "services"
    AIRLINES = "airlines"
    TRANSPORT = "transport"
    ACCOMMODATION = "accommodation"
    UTILITIES = "utilities"
    SHOPPING = "shopping"
    FINANCIAL = "financial"
    FURNITURE = "furniture"
    HARDWARE = "hardware"
    GROCERIES = "groceries"
    FUEL = "fuel"
    ENTERTAINMENT = "entertainment"
    SOFTWARE = "software"
    RESTAURANTS = "restaurants"
    ADVERTISING = "advertising"
    CASH = "cash"
    EDUCATION = "education"
    GOVERNMENT = "government"
