"""Retrieve information on exchange rates between currencies, buy and sell currencies.

Reference link: https://developer.revolut.com/docs/business/foreign-exchange
"""

# flake8: noqa: F401
from .endpoint import EndpointForeignExchangeSync, EndpointForeignExchangeAsync
