"""Retrieve information on existing team members of your organisation and invite new members.

This feature is available in the UK, US and the EEA.

This feature is not available in Sandbox.

Reference link: https://developer.revolut.com/docs/business/team-members
"""

# flake8: noqa: F401
from .endpoint import EndpointTeamMembersSync, EndpointTeamMembersAsync
