"""This module holds all GET counterparties endpoints."""

# flake8: noqa: F401
from .retrieve_counterparty import RetrieveCounterparty
from .retrieve_list_of_counterparties import RetrieveListOfCounterparties
