from enum import StrEnum


class EnumTransferReasonCode(StrEnum):
    """Transfer reason code."""

    ADVERTISING = "advertising"
    ADVISOR_FEES = "advisor_fees"
    BUSINESS_INSURANCE = "business_insurance"
    CONSTRUCTION = "construction"
    DELIVERY = "delivery"
    EDUCATION = "education"
    FAMILY = "family"
    FUND_INVESTMENT = "fund_investment"
    GOODS = "goods"
    HOMESEND = "homesend"
    HOTEL = "hotel"
    EXPORTS = "exports"
    INSURANCE_CLAIMS = "insurance_claims"
    INSURANCE_PREMIUM = "insurance_premium"
    LOAN_REPAYMENT = "loan_repayment"
    MEDICAL = "medical"
    OFFICE = "office"
    OTHER_FEES = "other_fees"
    PERSONAL_TRANSFER = "personal_transfer"
    PROPERTY_PURCHASE = "property_purchase"
    PROPERTY_RENTAL = "property_rental"
    ROYALTIES = "royalties"
    SERVICES = "services"
    SHARE_INVESTMENT = "share_investment"
    TAX = "tax"
    TRANSFER = "transfer"
    TRANSPORTATION = "transportation"
    TRAVEL = "travel"
    UTILITIES = "utilities"
    DONATIONS = "donations"
    FAMILY_SUPPORT = "family_support"
    SALARY = "salary"
