"""This module holds all POST transfers endpoints."""

# flake8: noqa: F401
from .create_transfer_to_another_account import CreateTransferToAnotherAccount
from .move_money_between_accounts import MoveMoneyBetweenAccounts
