"""This module holds all GET webhooks endpoints."""

# flake8: noqa: F401
from .retrieve_list_of_failed_webhooks import RetrieveListOfFailedWebhooks
from .retrieve_list_of_webhooks import RetrieveListOfWebhooks
from .retrieve_webhook import RetrieveWebhook
