"""Get the balances, full banking details, and other details of your business accounts.
Reference link: https://developer.revolut.com/docs/business/accounts
"""

# flake8: noqa: F401
from .endpoint import EndpointAccountsSync, EndpointAccountsAsync
