"""Move funds in the same currency between accounts of your business,
or make payments to your counterparties.

Reference link: https://developer.revolut.com/docs/business/transfers
"""

# flake8: noqa: F401
from .endpoint import EndpointTransfersSync, EndpointTransfersAsync
